<?php
/**
 * Created by PhpStorm.
 * User: usuario1
 * Date: 25/10/2017
 * Time: 01:02 PM
 */

require_once 'customList.class.php';
require_once 'lib/customlib/database/dao2/daoSql.php';

/**
 * Class customListDAO
 */
class customListDAO extends customList {

    protected $statusResponse;

    /**
     * customListDAO constructor.
     */
    public function __construct($ordered = true) {
        parent::customList($ordered);
    }


    /**
     * @param customListDAOItemIfc $item
     */
    public function add(customListDAOItemIfc $item){
        parent::add($item);
        //$this->list[$item->getListIndex()] = $item;
    }

    /**
     * Instancia la lista a partir de un objeto JSON
     *
     * @param $jsonObj
     * @return bool|customListDAO
     */
    public function instantiateFromJsonObject($jsonObj){
        $_ret = false;

        if ($jsonObj) {
            $statusResponse = statusResponse::getInstanceOk();
            $__className = $this->getItemClassName();

            foreach ($jsonObj as $item){
                $itemClass = new $__className();
                Cast::getInstantiatedEntityFromJsonObject($itemClass, $item);

                $this->add($itemClass);
            }

        } else {
            $statusResponse = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_ENTITY_INSTANTIATE_NOTFOUND);
        }

        $this->setStatusResponse($statusResponse);

        if (!$_ret)
            $_ret = clone $this;

        return $_ret;
    }

    private function getItemListFromClass($itemClass,$foreignKeyValue){
        try {
            $daoSql = new daoSql();
            $dbConn = $daoSql->dbConnect();

            if (!DbConfig::isError($dbConn)) {

                if ($foreignKeyValue) {
                    $query = static::getQueryForList($foreignKeyValue,$itemClass);
                                        
                    $daoResult = $daoSql->select($query);
                    
                    $statusResponse = $daoResult->getStatusResponse();
                    
                    if($statusResponse->isOk()) {
                        $it = $daoResult->getIterator();

                        foreach ($it as $row){
                            $item = $itemClass->instantiateFromResultset($row);
                            
                            $this->add($item);
                        }
                    }

                } else {
                    $statusResponse = statusResponse::getInstanceFromTableMsgCode( msgTable::ERROR_ENTITY_ID_NOTFOUND);
                }

            } else {
                $statusResponse = statusResponse::getInstanceForMDBError($dbConn);
            }

        } catch (Exception $e) {

            $statusResponse = statusResponse::getInstanceFromException($e);
        } finally {

            $this->setStatusResponse($statusResponse);
        }
    }

    /**
     * Instancia la lista a partir de un valor de la clave foranea
     *
     * @param $foreignKeyValue
     * @return statusResponse
     */
    public function getPlainInstance($foreignKeyValue){
        $__className = $this->getItemClassName();
        $itemClass = new $__className();

        $itemClass->setPlainMode();

        return $this->getItemListFromClass($itemClass,$foreignKeyValue);
    }

    /**
     * Instancia la lista a partir de un valor de la clave foranea
     *
     * @param $foreignKeyValue
     * @return statusResponse
     */
    public function getInstance($foreignKeyValue){
        $__className = $this->getItemClassName();
        $itemClass = new $__className();

        $itemClass->setAutoMode();

        return $this->getItemListFromClass($itemClass,$foreignKeyValue);
    }

    /**
     * Instancia la lista a partir de un valor de la clave foranea
     *
     * @param $foreignKeyValue
     * @return statusResponse
     */
    public function getDeepInstance($foreignKeyValue){
        $__className = $this->getItemClassName();
        $itemClass = new $__className();

        $itemClass->setDeepMode();

        return $this->getItemListFromClass($itemClass,$foreignKeyValue);
    }

    /**
     * Persiste la lista de items hacia la base de datos
     *
     * @return statusResponse
     */
    public function Persist($foreignKeyValue){
        $statusResponse = statusResponse::getInstanceOk();

        try {
            $transaction = dao2::BeginTransaction();

            $it = $this->getIterator();
            foreach ($it as $item) {

                if($item->isDeleted())
                    $_persistRet = $item->Delete($item->getPrimaryKeyValue());
                else{

                    $foreignKeyMethod = "set" . ucfirst($this->getForeignKey());
                    $item->$foreignKeyMethod($foreignKeyValue);

                    $_persistRet = $item->Persist();
                }

                if (!$_persistRet->isOk()) {
                    $statusResponse = $_persistRet;
                    break;
                }
            }

            if ($statusResponse->isOk()) {
                $transaction->Commit();
            }else {
                $transaction->Rollback();
            }
        }catch (Exception $e){
            $statusResponse = statusResponse::getInstanceFromTableMsgCode( msgTable::ERROR_ENTITY_PERSIST_FAIL);
            $statusResponse->setDebugInfo($e->getMessage());
        }

        return $statusResponse;
    }

    /** @return statusResponse */
    public function getStatusResponse() {return $this->statusResponse;}
    /** @param mixed $statusResponse */
    public function setStatusResponse($statusResponse) {$this->statusResponse = $statusResponse;}

    private function getQueryForList($foreignKeyValue, $itemClass){
        $tableName = $itemClass->tableName;
        $foreignKey = $this->getForeignKey();

        return "SELECT * FROM $tableName WHERE $foreignKey = $foreignKeyValue";
    }

}